package com.gic.store.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.commons.util.EntityUtil;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.constant.StrategyStoreFieldEnum;
import com.gic.store.dao.mapper.TabStoreStrategyMapper;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreExtendDTO;
import com.gic.store.dto.StoreStrategyDTO;
import com.gic.store.entity.TabStore;
import com.gic.store.entity.TabStoreExtend;
import com.gic.store.entity.TabStoreStrategy;
import com.gic.store.service.StoreExtendService;
import com.gic.store.service.StoreService;
import com.gic.store.service.StoreStrategyService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author guojx
 * @date 2019/7/4 9:47 AM
 */
@Service("storeStrategyService")
public class StoreStrategyServiceImpl implements StoreStrategyService{

    @Autowired
    private TabStoreStrategyMapper tabStoreStrategyMapper;

    @Autowired
    private StoreService storeService;
    @Autowired
    private StoreExtendService storeExtendService;

    @Override
    public int saveStoreGroupStrategy(StoreStrategyDTO storeStrategyDTO) {
        TabStoreStrategy storeStrategy = EntityUtil.changeEntityNew(TabStoreStrategy.class, storeStrategyDTO);
        tabStoreStrategyMapper.insert(storeStrategy);
        return storeStrategy.getStrategyId();
    }

    @Override
    public boolean isRepeatGroupStrategyName(TabStoreStrategy record) {
        Integer count = tabStoreStrategyMapper.countRepeatStrategyName(record);
        if (count != null && count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public TabStoreStrategy getStoreStrategyById(Integer strategyId) {
        return tabStoreStrategyMapper.selectByPrimaryKey(strategyId);
    }

    @Override
    public TabStoreStrategy getNextSort(int enterpriseId, double sort, int strategyType) {
        return tabStoreStrategyMapper.selectNextByEnterpriseIdAndSort(enterpriseId, sort, strategyType);
    }

    @Override
    public TabStoreStrategy getPrevSort(int enterpriseId, double sort, int strategyType) {
        return tabStoreStrategyMapper.selectPrevByStoreFieldIdAndSort(enterpriseId, sort, strategyType);
    }

    @Override
    public void setStoreStrategySort(int strategyId, int enterpriseId, int setValue, int strategyType) {
        if (setValue <= 1) {
            // 相当于置顶
            sortTopping(enterpriseId, strategyId, strategyType);
        } else {
            int start = setValue - 2;
            int size = 2;

            List<TabStoreStrategy> list = tabStoreStrategyMapper.listTwoElementExceptId(strategyId, start, size, enterpriseId, strategyType);
            if (CollectionUtils.isEmpty(list)) {
                // 置底
                sortBottom(enterpriseId, strategyId, strategyType);
            } else {
                Double sort;
                if (list.size() == 1) {
                    // 置底
                    sort = list.get(0).getSort() + 1d;
                } else {
                    sort = (list.get(0).getSort() + list.get(1).getSort()) / 2d;
                }
                updateSortById(strategyId, sort);
            }
        }
    }

    @Override
    public double getMinSort(int enterpriseId, int strategyType) {
        Double sort = tabStoreStrategyMapper.getMinSort(enterpriseId, strategyType);
        return sort == null ? 1d : sort;
    }

    @Override
    public void sortBottom(int enterpriseId, int strategyId, int strategyType) {
        Double sort = getMaxSort(enterpriseId, strategyType);
        updateSortById(strategyId, sort + 1d);
    }

    @Override
    public void sortTopping(int enterpriseId, int strategyId, int strategyType) {
        Double sort = getMinSort(enterpriseId, strategyType);
        updateSortById(strategyId, sort - 1d);
    }

    @Override
    public int deleteStoreGroupStrategy(int strategyId) {
        TabStoreStrategy storeStrategy = new TabStoreStrategy();
        storeStrategy.setStrategyId(strategyId);
        storeStrategy.setStatus(0);
        return tabStoreStrategyMapper.updateByPrimaryKeySelective(storeStrategy);
    }

    @Override
    public int modifyStoreGroupStrategy(StoreStrategyDTO storeStrategyDTO) {
        return tabStoreStrategyMapper.updateByPrimaryKeySelective(EntityUtil.changeEntityNew(TabStoreStrategy.class, storeStrategyDTO));
    }

    @Override
    public Page<TabStoreStrategy> listStoreStrategy(int enterpriseId, String search, int strategyType, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<TabStoreStrategy> list = tabStoreStrategyMapper.listStoreStrategy(enterpriseId, search, strategyType);
        return (Page<TabStoreStrategy>) list;
    }

    @Override
    public List<TabStoreStrategy> listStoreStrategy(int enterpriseId, int strategyType) {
        List<TabStoreStrategy> list = tabStoreStrategyMapper.listStoreStrategy(enterpriseId, null, strategyType);
        return list;
    }

    public double getMaxSort(int enterpriseId, int strategyType) {
        Double sort = tabStoreStrategyMapper.getMaxSort(enterpriseId, strategyType);
        return sort == null ? 1d : sort;
    }

    @Override
    public void updateSortById(int strategyId, double sort) {
        TabStoreStrategy storeStrategy = new TabStoreStrategy();
        storeStrategy.setStrategyId(strategyId);
        storeStrategy.setSort(sort);
        tabStoreStrategyMapper.updateByPrimaryKeySelective(storeStrategy);
    }

    @Override
    public String isHitStrategy(int storeId, int enterpriseId, int strategyType) {
        TabStore store = storeService.getById(storeId);
        return validIsHit(strategyType, store, null);
    }

    @Override
    public String isHitStrategy(StoreDTO storeDTO, int strategyType) {
        return validIsHit(strategyType, EntityUtil.changeEntityNew(TabStore.class, storeDTO), storeDTO.getStoreExtendList());
    }

    private String validIsHit(int strategyType, TabStore store, List<StoreExtendDTO> extendDTOList) {
        Map<Integer, String> extendMap = new HashMap<>(16);
        if (extendDTOList != null) {
            for (StoreExtendDTO extendDTO : extendDTOList) {
                extendMap.put(extendDTO.getStoreFieldId(), extendDTO.getValue());
            }
        }
        //命中值
        String result = "";
        List<TabStoreStrategy> strategyList = listStoreStrategy(store.getEnterpriseId(), strategyType);
        for (int num = 0, strategyLength = strategyList.size(); num < strategyLength; num ++) {
            //如果数值对上，则说明命中
            int validSize = 0;
            TabStoreStrategy strategy = strategyList.get(num);

            String targetValue = strategy.getTargetValue();
            String conditions = strategy.getConditions();
            int regionId = strategy.getStoreRegionId();
            String storeCreateType = strategy.getStoreCreateType();

            //valid
            if (store.getRegionId().intValue() != regionId) {
                continue;
            }

            validSize ++;

            JSONArray jsonArray = JSONArray.parseArray(conditions);
            for (int i = 0, length = jsonArray.size(); i < length; i ++) {
                JSONObject object = jsonArray.getJSONObject(i);
                String key = object.getString("key");
                String value = object.getString("value");
                if (StrategyStoreFieldEnum.RelatedBrand.getCode().equals(key)) {
                    if (!isBrandIdHitStrategy(store.getBrandIds(), value)) {
                        continue;
                    }
                } else if (StrategyStoreFieldEnum.StoreCity.getCode().equals(key)) {
                    if (!isBrandIdHitStrategy(store.getCityId(), value)) {
                        continue;
                    }
                } else if (StrategyStoreFieldEnum.StoreProvince.getCode().equals(key)) {
                    if (!isBrandIdHitStrategy(store.getProvinceId(), value)) {
                        continue;
                    }
                } else if (StrategyStoreFieldEnum.StoreCountry.getCode().equals(key)) {
                    if (!isBrandIdHitStrategy(store.getAreaId(), value)) {
                        continue;
                    }
                } else {
                    //单选或者多选
                    if (extendDTOList == null) {
                        TabStoreExtend storeExtend = storeExtendService.getStoreExtendByStoreIdAndFieldId(store.getStoreId(), Integer.parseInt(key));
                        if (storeExtend != null && !isBrandIdHitStrategy(storeExtend.getValue(), value)) {
                            continue;
                        }
                    } else {
                        String storeFieldValue = extendMap.get(Integer.parseInt(key));
                        if (StringUtils.isNotBlank(storeFieldValue) && !isBrandIdHitStrategy(storeFieldValue, value)) {
                            continue;
                        }
                    }
                }

                validSize ++;
            }
            if (strategyType == StoreGroupConstant.STORE_STRATEGY_TYPE) {
                //如果是门店启用状态策略，需要验证门店创建方式
                if (!isBrandIdHitStrategy(store.getCreateType().toString(), storeCreateType)) {
                    continue;
                }
                validSize ++;
            }

            int hitSize;
            if (strategyType == StoreGroupConstant.STORE_STRATEGY_TYPE) {
                hitSize = jsonArray.size() + 2;
            } else {
                hitSize = jsonArray.size() + 1;
            }
            //命中
            if (validSize == hitSize) {
                result = targetValue;
                break;
            }
        }
        return result;
    }

    private boolean isBrandIdHitStrategy(String brandIds, String value) {
        String[] values = value.split(",");
        for (String str : values) {
            if (brandIds.contains(str)) {
                return true;
            }
        }
        return false;
    }

}
